﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Rendering;
using Vezbe5i6.Models;
using Vezbe5i6.Repository;

namespace Vezbe5i6.Controllers
{
    public class ProductController : Controller
    {
        readonly IProductRepository _proRepositry;
        readonly ICategoryRepository _catRepositry;
        readonly string _imageFolderPath;
        public ProductController(IProductRepository proRepositry, ICategoryRepository catRepositry, IWebHostEnvironment env)
        {
            _proRepositry = proRepositry;
            _catRepositry = catRepositry;
            //podesavanje putanje ka folderu za slike
            _imageFolderPath = env.ContentRootPath + @"\wwwroot\images\";
        }

        public IActionResult Index([FromQuery] string filter, [FromQuery] int[] categories)
        {
            //ako je filter null setovati da bude prazan string
            if (filter == null)
                filter = "";
            var cats = _catRepositry.GetAll();
            var product = new List<Product>();

            //ako su filter i filter kategorija prazni onda uzmi sve proizvode u suprotnom odradi filter
            if (string.IsNullOrEmpty(filter) && !categories.Any())
                product = _proRepositry.GetAll().ToList();
            else
                product = _proRepositry.GetWithFilter(filter, categories).ToList();

            //kreiraj spisak kategorija za filter i podesi sta je selektovano a sta nije
            var categoryFilter = cats.Select(p => new CategoryFilter
            {
                Id = p.CategoryId,
                Name = p.Name,
                Selected = categories.Contains(p.CategoryId)
            });
            //kreiraj view model
            var viewModel = new ProductListView
            {
                CategoryFilter = categoryFilter,
                Products = product,
                Filter = filter
            };

            return View(viewModel);
        }
        public IActionResult Details(int id)
        {
            //ucitaj proizvod iz repozitorijuma
            var product = _proRepositry.GetSingle(id);
            //ako proiyvod ne postoji vrati na pregled proiyvoda
            if (product == null)
            {
                return RedirectToAction("index");
            }
            //ucitaj kategoriju za taj proizvod
            product.Category = _catRepositry.GetSingle(product.Category?.CategoryId ?? 0);

            return View(product);
        }
        public IActionResult Create()
        {
            //ucitaj kategorije za select listu
            var categories = _catRepositry.GetAll();
            //setuj u viewdata podatke za select listu kategorija
            ViewData["categories"] = new SelectList(categories, "CategoryId", "Name");

            return View("Edit", new Product());
        }
        public IActionResult Edit(int id)
        {
            var categories = _catRepositry.GetAll();
            ViewData["categories"] = new SelectList(categories, "CategoryId", "Name");

            return View(_proRepositry.GetSingle(id));
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult Edit(Product product)
        {
            try
            {
                //ukloni validaciju iz dete objekta Category kako ne bi vracao gresku da su code i name obavezni
                ModelState.Remove("Category.Name");
                ModelState.Remove("Category.Code");
                //provera da li je model valida tj. da li su svi kriterijumi zadovoljeni koji su definisani u modelu
                if (!ModelState.IsValid)
                {
                    var categories = _catRepositry.GetAll();
                    ViewData["categories"] = new SelectList(categories, "CategoryId", "Name");
                    return View(product);
                }


                //cuvanje proizvoda
                var result = _proRepositry.Save(product);
                //ako je save metoda vratila -10(kao primer) vrati gresku da sifra postoji na drugom proizvodu
                if (result == -10)
                {
                    //kako bi se vratila forma potrebno je ponovo ucitati kategorije u viewdata
                    var categories = _catRepositry.GetAll();
                    ViewData["categories"] = new SelectList(categories, "CategoryId", "Name");
                    //setuj error na sifri proizvoda kako bi se pokazao
                    ModelState.AddModelError("Code", "Šifra već postoji u bazi");

                    return View(product);
                }

                //cuvanje slike
                if (product.NewImage != null && product.NewImage.Length > 0)
                {
                    using (var stream = System.IO.File.Create(_imageFolderPath + product.NewImage.FileName))
                    {
                        product.NewImage.CopyTo(stream);
                    }
                    if (!string.IsNullOrEmpty(product.ImageName))
                    {
                        DeleteImage(product.ImageName);
                    }
                    product.ImageName = product.NewImage.FileName;
                }

                return RedirectToAction(nameof(Index));
            }
            catch
            {
                var categories = _catRepositry.GetAll();
                ViewData["categories"] = new SelectList(categories, "CategoryId", "Name");

                return View(product);
            }
        }
        public IActionResult Delete(int id)
        {
            //ucitaj detalje proizvoda kako bi imao podatak o slici kako bi se mogla obrisati
            var product = _proRepositry.GetSingle(id);
            if (product == null)
            {
                return RedirectToAction("Index");
            }

            //obrisi proizvod
            _proRepositry.Delete(id);
            //obrisi sliku
            if (!string.IsNullOrEmpty(product.ImageName))
            {
                DeleteImage(product.ImageName);
            }

            return RedirectToAction("Index");
        }
        private void DeleteImage(string imageName)
        {
            //fpormiraj full path
            var fullImagePath = _imageFolderPath + imageName;
            //proveri da li slika postoji
            //dodatna provera da ako se zavrsava na _nd.jpg se ne brise jer su to slike u sklopu vezbe koje trebaju uvek da ostanu
            if (System.IO.File.Exists(fullImagePath) && !imageName.EndsWith("_nd.jpg"))
            {
                //brisanje slike sa diska
                System.IO.File.Delete(fullImagePath);
            }
        }
    }
}
